﻿using System;
using System.IO;
using System.Collections.Generic;

// http://www.frank-t-clark.com/Professional/Papers/STemplate/STemplate.html
namespace STemplate
{
	struct Section
	{
		public string save;	// first level null
		public string working;
		public string tag;	// first level null
	}

	/// <summary>
	/// Simple String Template class
	/// </summary>
	class S_Template
	{
		Stack<Section> section = new Stack<Section> ();
		Section previous_section;
		Section current_section;

		// Default constructor sufficient

		// Public Methods

		/// <summary>
		/// Read a new file as the template
		/// </summary>
		/// <param name="filename">Template file read location</param>
		public void Read (string filename)
		{
			Clear ();
			current_section.working = File.ReadAllText (filename);
		}

		/// <summary>
		/// Begin a new string as the template
		/// </summary>
		/// <param name="str">Template string</param>
		public void Begin (string str)
		{
			Clear ();
			current_section.working = str;
		}

		/// <summary>
		/// Discard all template working data
		/// </summary>
		public void Clear ()
		{
			// remove any previous data
			section.Clear ();
			previous_section.save = null;
			previous_section.working = null;
			previous_section.tag = null;
			current_section.save = null;
			current_section.working = null;
			current_section.tag = null;
		}

		/// <summary>
		/// Replace a template element in the current section
		/// </summary>
		/// <param name="element_text">Template element to be replaced</param>
		/// <param name="replace_text">Replacement value</param>
		public void Replace (string element_text, string replace_text)
		{
			// was there a Begin
			if (null == current_section.working)
				return;
			current_section.working = current_section.working.Replace (element_text, replace_text);
		}

		/// <summary>
		/// Create a fresh working copy of a section of the template
		/// (inserts any previous matching SectionBegin)
		/// </summary>
		/// <param name="section_tag">Template section name</param>
		public void SectionBegin (string section_tag)
		{
			// was there a Begin
			if (null == current_section.working)
				return;
			// is this the same section_tag
			if (current_section.tag == section_tag)
			{
				// if any SectionBegin previously
				SectionInsert ();
				current_section.working = current_section.save;
				return;
			}
			// is there actually a section
			int start = current_section.working.IndexOf (section_tag);
			if (0 > start)
				return;
			start += section_tag.Length;
			int end = current_section.working.IndexOf (section_tag, start);
			if (0 > end)
				return;
			// not the first SectionBegin
			if (null != previous_section.working)
				section.Push (previous_section);
			previous_section = current_section;
			current_section.tag = section_tag;
			current_section.save = previous_section.working.Substring (start, end - start);
			start -= section_tag.Length;
			previous_section.working = previous_section.working.Remove (start, end - start);
			current_section.working = current_section.save;
		}

		/// <summary>
		/// Finalize the current section in the working copy of the template
		/// (inserts any previous matching SectionBegin)
		/// </summary>
		public void SectionEnd ()
		{
			// was there a SectionBegin
			if (null == previous_section.working)
				return;
			SectionInsert ();
			int start = previous_section.working.IndexOf (current_section.tag);
			if (0 > start)
				return;
			previous_section.working = previous_section.working.Remove (start, current_section.tag.Length);
			current_section = previous_section;
			if (0 == section.Count)
			{
				previous_section.save = null;
				previous_section.tag = null;
				previous_section.working = null;
				return;
			}
			previous_section = section.Pop ();
		}

		//
		// Copy the previous SectionBegin if any to the previous working copy of the template
		//
		private void SectionInsert ()
		{
			// was there a SectionBegin
			if (null == previous_section.working)
				return;
			int start = previous_section.working.IndexOf (current_section.tag);
			if (0 > start)
				return;
			previous_section.working = previous_section.working.Insert (start, current_section.working);
		}

		/// <summary>
		/// Remove the entire section from the working copy of the template
		/// instead of any Section Begin
		/// </summary>
		/// <param name="section_tag">Template section name</param>
		public void SectionRemove (string section)
		{
			int start = current_section.working.IndexOf (section);
			if (0 > start)
				return;
			int end = current_section.working.IndexOf (section, start + 1);
			if (0 > end)
				return;
			end += section.Length;
			current_section.working = current_section.working.Remove (start, end - start);
		}

		/// <summary>
		/// Return the current working section of the template
		/// </summary>
		public string End ()
		{
			string str = current_section.working;
			Clear ();
			return str;
		}

		/// <summary>
		/// Write the current working section of the template to a file
		/// then discard all remaining template working data
		/// </summary>
		/// <param name="filename">Finished file write location</param>
		public void Write (string filename)
		{
			File.WriteAllText (filename, current_section.working);
			Clear ();
		}

		/// <summary>
		/// Finish all open sections
		/// </summary>
		public void Finish ()
		{
			// was there a SectionBegin
			while (null != previous_section.working)
				SectionEnd ();
			return;
		}

	}
}
